//
//  LSConnectionDetails.h
//  Lightstreamer client for iOS UCA
//

#import <Foundation/Foundation.h>


/**
 Used by <LSLightstreamerClient> to provide a basic connection properties object.
 
 This object contains the configuration settings needed to connect to a Lightstreamer Server.
 
 An instance of this class is attached to every <LSLightstreamerClient> as <[LSLightstreamerClient connectionDetails]>.
 
 @see LSLightstreamerClient
 */
@interface LSConnectionDetails : NSObject


/**
 @brief Name of the Adapter Set (which defines the Metadata Adapter and one or several Data Adapters) mounted on Lightstreamer Server that supply all the items 
 used in this application.
 
 An Adapter Set defines the Metadata Adapter and one or several Data Adapters. It is configured on the server side through an `adapters.xml` file;
 the name is configured through the `id` attribute in the <adapters_conf> element. The default Adapter Set, configured as `DEFAULT` on the Server.
 
 **Lifecycle:** the Adapter Set name should be set on the <[LSLightstreamerClient connectionDetails]> object before calling the <[LSLightstreamerClient connect]> method. 
 However, the property can be changed at any time: the supplied value will be used for the next time a new session is requested to the server.
 
 This setting can also be specified in the <LSLightstreamerClient> constructor. A nil value is equivalent to the `DEFAULT` name.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `adapterSet`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 */
@property (nonatomic, copy, nullable) NSString *adapterSet;


/**
 @brief Configured address of Lightstreamer Server.
 
 Note that the addresses specified must always have the http: or https: scheme. In case WebSockets are used, the specified scheme is internally converted to
 match the related WebSocket protocol (i.e. http becomes ws while https becomes wss). If no server address is supplied the client will be unable to connect.
 
 Note: WSS/HTTPS is an optional feature, available depending on Edition and License Type.
 
 **Lifecycle:** this property can be changed at any time. If changed while connected, it will be applied when the next session creation request is issued.
 This setting can also be specified in the LightstreamerClient constructor. A nil value can also be used, to restore the default value. An IPv4 or IPv6 can also 
 be used in place of a hostname. Some examples of valid values include:
 
 - http://push.mycompany.com

 - http://push.mycompany.com:8080
 
 - http://79.125.7.252
 
 - http://[2001:0db8:85a3:0000:0000:8a2e:0370:7334]
 
 - http://[2001:0db8:85a3::8a2e:0370:7334]:8080
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `serverAddress`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 
 @exception NSException if the given address is not valid.
 */
@property (nonatomic, copy, nullable) NSString *serverAddress;

/**
 @brief Server address to be used to issue all requests related to the current session.
 
 In fact, when a Server cluster is in place, the Server address specified through <serverAddress> can identify various Server instances; in order to ensure
 that all requests related to a session are issued to the same Server instance, the Server can answer to the session opening request by providing an address which 
 uniquely identifies its own instance. When this is the case, this address is returned as the value; otherwise, nil is returned.
 
 Note that the addresses will always have the http: or https: scheme. In case WebSockets are used, the specified scheme is internally converted to match the
 related WebSocket protocol (i.e. http becomes ws while https becomes wss). 
 
 Note: server clustering is an optional feature, available depending on Edition and License Type.

 **Lifecycle:** the value of this property is meaningful only when a session is currently active.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `serverInstanceAddress`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 */
@property (nonatomic, readonly, nullable) NSString *serverInstanceAddress;

/**
 @brief Instance name of the Server which is serving the current session.
 
 To be more precise, each answering port configured on a Server instance (through a &lt;http_server&gt; or &lt;https_server&gt; element in the Server configuration file) 
 can be given a different name; the name related to the port to which the session opening request has been issued is returned.
 
 Note that each rebind to the same session can, potentially, reach the Server on a port different than the one used for the previous request, 
 depending on the behavior of intermediate nodes. However, the only meaningful case is when a Server cluster is in place and it is configured in such a way 
 that the port used for all `bind_session` requests differs from the port used for the initial `create_session` request.
 
 Note: server clustering is an optional feature, available depending on Edition and License Type.

 **Lifecycle:** if a session is not currently active, nil is returned; soon after a session is established, the value will become available.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `serverSocketName`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 */
@property (nonatomic, readonly, nullable) NSString *serverSocketName;

/**
 @brief IP address of this client as seen by the Server which is serving the current session as the client remote address 
 (note that it may not correspond to the client host; for instance it may refer to an intermediate proxy).
 
 If, upon a new session, this address changes, it may be a hint that the intermediary network nodes handling the connection have changed, hence the network
 capabilities may be different. The library uses this information to optimize the connection.
 
 Note that in case of polling or in case rebind requests are needed, subsequent requests related to the same session may, in principle,
 expose a different IP address to the Server; these changes would not be reported.
 
 **Lifecycle:** if a session is not currently active, nil is returned; soon after a session is established, the value may become available; but it is possible
 that this information is not provided by the Server and that it will never be available.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `clientIp`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 */
@property (nonatomic, readonly, nullable) NSString *clientIp;

/**
 @brief ID associated by the server to this client session.
 
 **Lifecycle:** the value of this property is meaningful only when a session is currently active.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `sessionId`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 */
@property (nonatomic, readonly, nullable) NSString *sessionId;

/**
 @brief Username to be used for the authentication on Lightstreamer Server when initiating the session.
 
 The Metadata Adapter is responsible for checking the credentials (username and password). If no username is supplied, no user information will be sent at session
 initiation. The Metadata Adapter, however, may still allow the session. 
 
 **Lifecycle:** the username should be set on the <[LSLightstreamerClient connectionDetails]> object before
 calling the <[LSLightstreamerClient connect]> method. However, the property can be changed at any time: the supplied value will be used for the next time a new session
 is requested to the server.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `user`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 
 @see setPassword:
 */
@property (nonatomic, copy, nullable) NSString *user;

/**
 @brief Setter method that sets the password to be used for the authentication on Lightstreamer Server when initiating the session.
 
 The Metadata Adapter is responsible for checking the credentials (username and password).
 If no password is supplied, no password information will be sent at session initiation. The Metadata Adapter, however, may still allow the session.
 
 **Lifecycle:** the username should be set on the <[LSLightstreamerClient connectionDetails]> object before calling the <[LSLightstreamerClient connect]> method. However,
 the property can be changed at any time: the supplied value will be used for the next time a new session is requested to the server.
 
 NOTE: The password string will be stored in the current instance. That is necessary in order to allow automatic reconnection/reauthentication for fail-over.
 For maximum security, avoid using an actual private password to authenticate on Lightstreamer Server; rather use a session-id originated by your web/application 
 server, that can be checked by your Metadata Adapter.
 
 **Related notifications:** a change to this setting will be notified through a call to <[LSClientDelegate client:didChangeProperty:]> with argument `password`
 on any <LSClientDelegate> listening to the related <LSLightstreamerClient>.
 
 @param password The password to be used for the authentication on Lightstreamer Server. The password can be nil.
 
 @see user
 */
- (void) setPassword:(nullable NSString *)password;


@end
